// UrsOTA
//
// Autor: http://UllisRoboterSeite.de
// Doku:  http://bienonline.magix.net/public/esp8266-ota.html
// Created: 2017-04-26
//
// Version 1.0 (2017-01-11)
// - Basis-Version
// Version 1.1 (2017-01-12)
// - nur intern benutzte Variablen als 'static' definiert
// - Methode 'begin': Typ des Parameters 'OutDevide' von '&Print' auf '*Print' gendert.
//   Das erlaubt die bergabe von Null fr Systeme, in denen keine Ausgabe erfolgen soll.
// Version 1.2 (2017-04-26)
// - Passwort zu den Constructor-Paramtern hinzugefgt.
// - Typ von Constructor-Paramter "HostName" von 'String' -> 'const char*' gendert.
// Version 1.3 (2017-06-03)
// - Zweite Version von begin mit Strings als Parameter
// Version 1.4 (2017-11-25)
// - Definierbare Handler-Funktionen fr Start, Ende Fortschritt und Fehler hinzugefgt.
// - Konstruktoren mit Angabe von Hostname und Ausgabe-Gert hinzugefgt.
// Version 1.5 (2017-12-05)
// - Bei der Initialisierung wird der Hostname mit ausgegeben.
// Version 1.6 (2018-01-31)
// - Meldungen in deutsch.
// Version 1.7 (2018-07-23)
// - UrsOTA.begin(String) war mehrdeutig
// Version 1.8 (2018-09-11)
// - Text-Konstanten ins Flash gelegt (printf_P benutzt)


// Ermglicht die vereinfachte Benutzung der ArduinoOTA-Klasse.
// - Hostname, Port und Ausgabe-Gert knnen optional der Methode begin mitgegeben werden.
//   Die Voreinstellungen "" und 0 fhren zu "esp8266-<ChipID>" und 8266.
//   Fr OutDevice ist Serial voreingestellt.
// - onStart, onEnd, onProgress, onError sind mit Funktionen hinterlegt, 
//   die eine Ausgabe ber eine von Print abgeleitete Klasse ermglichen.
//   Die Voreinstellung ist Serial.

#ifndef _URSOTA_h
#define _URSOTA_h

#if defined(ARDUINO) && ARDUINO >= 100
	#include "arduino.h"
#else
	#include "WProgram.h"
#endif
#include <ArduinoOTA.h>


typedef std::function<void(void)> UrsOtaHandlerFunction;
typedef std::function<void(ota_error_t)> UrsOtaHandlerFunction_Error;
typedef std::function<void(uint8_t)> UrsOtaHandlerFunction_Progress;

class UrsOTAClass
{public:
  // Initialisiert den OTA-Service.
  // begin() kann nur einmal aufgerufen werden. ArduinoOTA verhindert, das Folgeaufrufe wirksam sind.
  void begin(const char *HostName, const int Port, const char *PassWord = NULL, Print *OutDevide = &Serial);
  void begin(const String HostName, const int Port, const String PassWord = "", Print *OutDevide = &Serial);
  void begin(const char *HostName = NULL, Print *OutDevide = &Serial);
  void begin(const String HostName ="", Print *OutDevide = &Serial);

  // Gibt eine Callback-Funktion an, die ...
  void onStart(UrsOtaHandlerFunction fn); //... beim OTA-Start ausgefhrt wird.
  void onEnd(UrsOtaHandlerFunction fn); //... die beim Ende von OTA ausgefhrt wird.
  void onProgress(UrsOtaHandlerFunction_Progress fn); // ... bei Fortschritt ausgefhrt wird (10%-Schritte).
  void onError(UrsOtaHandlerFunction_Error fn); // ... bei einem Upload-Fehler ausgefhrt wird.

  // Ruft den OTA-Service ab.
  void handle() {ArduinoOTA.handle();};

private:
  UrsOtaHandlerFunction _start_callback = NULL;       // Callback fr Start
  UrsOtaHandlerFunction _end_callback = NULL;         // Callback fr Ende
  UrsOtaHandlerFunction_Error _error_callback = NULL; // Callback fr Fehlerfall
  UrsOtaHandlerFunction_Progress _progress_callback = NULL; // Callback fr Fortschritt
  Print * _OutDevice = NULL; // Device zur Meldungsausgabe
  uint8_t lastPercentage = 255; // irgendein, nicht durch 10 teilbarer Wert
};

extern UrsOTAClass UrsOTA;

#endif

